
import hmac
from uuid import uuid4
from hashlib import sha256
from datetime import timedelta
from django.utils.timezone import now
from django.core.cache import cache
from .conf import TOKEN_TYPES


def _getkey(token):
    return 'apitoken-{}'.format(token)


class Token(object):

    def __init__(self, user, typ, token=None, expires=None):
        self.user = user
        self.typ = typ
        if token is None:
            self.token = hmac.new(uuid4().bytes, digestmod=sha256).hexdigest()
        else:
            self.token = token
        self.life = [t['life'] for t in TOKEN_TYPES if t['key'] == self.typ][0]
        self.expires = expires

    @property
    def is_valid(self):
        return self.user.is_active and self.expires >= now()

    def delete(self):
        cache.delete(_getkey(self.token))

    def save(self):
        self.expires = now() + timedelta(seconds=self.life)
        data = {
            'user': self.user,
            'typ': self.typ,
            'expires': self.expires,
        }
        cache.set(_getkey(self.token), data, self.life)


class TokenAPI(object):

    def get(self, token):
        """
        Returns a valid token if it exists else False
        """

        data = cache.get(_getkey(token))
        if data:
            token = Token(token=token, **data)
            if token.is_valid:
                token.save()
                return token

    def create(self, user, typ=None):
        """
        Creates a new token for the given user and type
        """

        if typ is None:
            typ = TOKEN_TYPES[0]['key']

        if typ not in [t['key'] for t in TOKEN_TYPES]:
            raise Exception('Invalid type {}'.format(typ))

        token = Token(user=user, typ=typ)
        token.save()
        return token


for typ in TOKEN_TYPES:
    setattr(TokenAPI, typ['key'].replace('-', '_').upper(), typ['key'])



from datetime import datetime
from mock import patch
from django.test import TestCase
from django.contrib.auth import get_user_model
from .api import TokenAPI


class APITestCase(TestCase):
    def setUp(self):
        self.user = get_user_model().objects.create()
        self.api = TokenAPI()

    def test_attributes(self):
        self.assertEqual(TokenAPI.STANDARD, 'standard')
        self.assertEqual(TokenAPI.APP, 'app')
        self.assertEqual(TokenAPI.OTP_TMP, 'otp-tmp')
        self.assertEqual(TokenAPI.OTP_VERIFIED, 'otp-verified')

    @patch('apitoken.api.now')
    def test_create(self, mnow):
        mnow.return_value = datetime(2011, 6, 21, 9, 43, 33)
        token = self.api.create(self.user, typ=TokenAPI.STANDARD)
        self.assertEqual(token.user, self.user)
        self.assertEqual(token.typ, 'standard')
        self.assertEqual(token.life, 60 * 60 * 4)
        self.assertEqual(token.expires, datetime(2011, 6, 21, 13, 43, 33))

    @patch('apitoken.api.now')
    def test_get(self, mnow):
        # Create initial token
        mnow.return_value = datetime(2011, 6, 21, 9, 43, 33)
        token = self.api.create(self.user, typ=TokenAPI.STANDARD)

        # Validate when still valid
        mnow.return_value = datetime(2011, 6, 21, 10, 43, 33)
        token = self.api.get(token.token)
        self.assertEqual(token.user, self.user)
        self.assertEqual(token.typ, 'standard')
        self.assertEqual(token.life, 60 * 60 * 4)
        self.assertEqual(token.expires, datetime(2011, 6, 21, 14, 43, 33))

        # Validate expired
        mnow.return_value = datetime(2011, 6, 21, 14, 43, 34)
        token = self.api.get(token.token)
        self.assertIsNone(token)

    def test_delete(self):
        token = self.api.create(self.user, typ=TokenAPI.STANDARD)
        token.delete()
        token = self.api.get(token.token)
        self.assertIsNone(token)



# Django settings for django-token project.
import os

DEBUG = True
TEMPLATE_DEBUG = DEBUG
DEBUG_PROPAGATE_EXCEPTIONS = True

ALLOWED_HOSTS = ['*']

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)

MANAGERS = ADMINS

if 'TRAVIS' in os.environ:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'NAME': 'travis',
            'USER': 'postgres',
            'PASSWORD': '',
            'HOST': 'localhost',
            'PORT': '',
        }
    }

else:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'NAME': 'docker',
            'USER': 'docker',
            'PASSWORD': 'docker',
            'HOST': 'docker',
            'PORT': '',
        }
    }

TIME_ZONE = 'Europe/London'
LANGUAGE_CODE = 'en-uk'

SITE_ID = 1

USE_I18N = True
USE_L10N = True

MEDIA_ROOT = ''
MEDIA_URL = ''

STATIC_URL = '/static/'
STATIC_ROOT = ''

SECRET_KEY = 'u@x-aj9(hoh#rb-^ymf#g2jx_hp0vj7u5#b@ag1n^seu9e!%cy'

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'apitoken',
)

AUTH_USER_MODEL = 'auth.User'

CELERY_ALWAYS_EAGER = True
CELERY_EAGER_PROPAGATES_EXCEPTIONS = True



from django.conf import settings

TOKEN_TYPES = getattr(settings, 'TOKEN_TYPES', (
    {
        'key': 'standard',
        'label': 'Standard',
        'life': 60 * 60 * 4,
    },
    {
        'key': 'app',
        'label': 'App',
        'life': 60 * 60 * 24 * 31,
    },
    {
        'key': 'otp-tmp',
        'label': 'OTP Temp',
        'life': 60 * 10,
    },
    {
        'key': 'otp-verified',
        'label': 'OTP Verified',
        'life': 60 * 60 * 4,
    }
))
